inventory = {
	maxItems: 20,
	itemsPerLine: 5,
	items: [],
	x: 624,
	y: 0,
	wetnessRegen: .05,
	wetnessIncrease: .8,
	qualities: [
		'tattered',
		'second-rate',
		'ordinary',
		'sturdy',
		'impervious'
	],
	init: function(){
		for(var i=0;i<inventory.maxItems;i++){
			inventory.items[i]=null;
		}
		
		inventory.selectedItemIndex = null;
	},
	getEquippedWetness: function(){
		var totalWetness=0;
		
		var equippedCount = 0;
		
		for(var i=0;i<inventory.items.length;i++){
			//reduce wetness
			var item = inventory.items[i];
			if(item!==null && item.equipped){
				totalWetness+=item.wetness;
				equippedCount++;
			}
		}
		
		if(equippedCount===0){
			return 0;
		}
		
		return totalWetness/equippedCount;
	},
	updateWetness: function(step){
		var inWater = player.inWater();
		
		for(var i=0;i<inventory.items.length;i++){
			//reduce wetness
			var item = inventory.items[i];
			 
			if(item!==null){ 
				if(inWater){
					var equippedMod = .5;
					if(item.equipped){
						equippedMod = 1;
					}
					
					item.wetness = Math.min(1,
						item.wetness+inventory.wetnessIncrease*(1-item.waterResistance)*equippedMod
					);
				}else{
					//add wetness
					item.wetness = Math.max(0,item.wetness-inventory.wetnessRegen);
				}
			}
		}
	},
	getTotalColdResistance:function(){
		var invertedProtections = {
			"hat":1,
			"jacket":1,
			"pants":1,
			"left glove":1,
			"right glove":1,
			"left boot":1,
			"right boot":1
		}
		
		for(var i=0;i<inventory.items.length;i++){
			var item = inventory.items[i];
			if(item !== null && item.equipped && item.isClothing){
				invertedProtections[item.equipType] *= 1-item.coldResistance;
			}
		}
		
		var totalProtection = (
			(1.5 * (1-invertedProtections['left boot']))+
			(1.5 * (1-invertedProtections['right boot']))+
			(1-invertedProtections['left glove'])+
			(1-invertedProtections['right glove'])+
			(1-invertedProtections['pants'])+
			(1-invertedProtections['jacket'])+
			(1-invertedProtections['hat'])
		)/8;
		
		return totalProtection;
	},
	
	getTotalWindResistance:function(){
		var invertedProtections = {
			"hat":1,
			"jacket":1,
			"pants":1,
			"left glove":1,
			"right glove":1,
			"left boot":1,
			"right boot":1
		}
		
		for(var i=0;i<inventory.items.length;i++){
			var item = inventory.items[i];
			if(item !== null && item.equipped && item.isClothing){
				invertedProtections[item.equipType] *= 1-item.windResistance;
			}
		}
		
		var totalProtection = (
			(1-invertedProtections['left boot'])+
			(1-invertedProtections['right boot'])+
			(1-invertedProtections['left glove'])+
			(1-invertedProtections['right glove'])+
			(1-invertedProtections['pants'])+
			(1-invertedProtections['jacket'])+
			(2*(1-invertedProtections['hat']))
		)/8;
		
		return totalProtection;
	},
	inventoryCount: function(){
		var count=0;
		for(var i=0;i<inventory.items.length;i++){
			if(inventory.items[i]!==null){
				count++;
			}
		}
		return count;
	},
	draw: function(ctx,img){
		for(var i=0;i<inventory.items.length;i++){
			var x=i%inventory.itemsPerLine;
			var y=Math.floor(i/inventory.itemsPerLine);

			var margin = 4;
			
			ctx.fillStyle = "rgba(100,100,100,1)";
			ctx.fillRect(inventory.x+x*64+margin,inventory.y+y*32+margin,64-margin,32-margin);
			
			if(inventory.items[i]!==null){
			
				if(inventory.items[i].equipped){
					ctx.fillStyle = "rgba(0,200,0,1)";
					ctx.fillRect(inventory.x+x*64+margin,inventory.y+y*32+margin,64-margin,32-margin);
				}
			
				inventory.items[i].drawInventory(ctx,img,x*64+inventory.x,y*32+inventory.y);
				
				var wetness = inventory.items[i].wetness;
								
				//draw wetness
				ctx.fillStyle = "rgba(0,0,255,.5)";
				ctx.fillRect(inventory.x+x*64+margin,inventory.y+y*32+margin+(1-wetness)*(32-margin),64-margin,wetness*(32-margin));
				
			}
			
			if(i===inventory.selectedItemIndex){
				ctx.strokeStyle = "rgba(255,255,255,.9)";
				ctx.lineWidth = 4;
				ctx.strokeRect(inventory.x+x*64+margin,inventory.y+y*32+margin,64-margin,32-margin);
				
				var name = inventory.items[i].getName();
				var description = inventory.items[i].getDescription();
				
				rdetail.setItemDetail(name,description);
			}
		}
	},
	
	//attempts to put item in inventory and returns true if successful
	pickUp: function(item){
		for(var i=0;i<inventory.items.length;i++){
			if(inventory.items[i]===null){
				inventory.items[i]=item;
				rconsole.log("You pick up a "+item.getName()+".");
				return true;
			}
		}
		return false;
	},
	pickUpNext:function(){
		var tile = player.getTile();
		var items = tile.items;
		var item;
		for(var i=0;i<items.length;i++){
			item = items[i];
			if(item!== null){
				if(inventory.pickUp(item)){
					item.previouslyPickedUp = true;
					
					//remove from ground
					items[i]=null;
				}
				return;
			}
		}
	},
	use: function(){
		inventory.useSelectedItem();
	},
	useSelectedItem: function(item){
		if(inventory.selectedItemIndex!==null){
			if(inventory.items[inventory.selectedItemIndex]!==null){
				//use and remove if necessary
				if(inventory.items[inventory.selectedItemIndex].use()){
					inventory.items[inventory.selectedItemIndex]=null;
					inventory.selectedItemIndex = null;
					
					rdetail.clear();
				}
			}else{
				throw("null item");
			}
		}
	},
	itemRoll:function(qualityBonus){
		var item = null;
	
		var r = Math.random();
		var compare = 0;
		
		//FOOD
		if(r<(compare+=.15)){
			item = new Mushroom(qualityBonus);
		}else if(r<(compare+=.08)){
			item = new Berry(qualityBonus);
		}else if(r<(compare+=.03)){	
			item = new Bread(qualityBonus);
		}else if(r<(compare+=.04)){
			item = new Hat(qualityBonus);
		}else if(r<(compare+=.04)){
			item = new Jacket(qualityBonus);
		}else if(r<(compare+=.04)){
			item = new Pants(qualityBonus);
		}else if(r<(compare+=.08)){
			item = new Glove(qualityBonus);
		}else if(r<(compare+=.08)){
			item = new Boot(qualityBonus);
		}else if(r<(compare+=.01)){
			item = new Compass(qualityBonus);
		}else if(r<(compare+=.015)){
			item = new Watch(qualityBonus);
		}else if(r<(compare+=.02)){
			item = new Shotgun(qualityBonus);
		}else if(r<(compare+=.03)){
			item = new Ammo(qualityBonus);
		}else if(r<(compare+=.02)){
			item = new Key(qualityBonus);
		}else if(r<(compare+=.03)){
			item = new Blanket(qualityBonus);
		}else if(r<(compare+=.02)){
			item = new SleepingBag(qualityBonus);
		}else if(r<(compare+=.01)){
			item = new Tent(qualityBonus);
		}else if(r<(compare+=.01)){
			item = new Binoculars(qualityBonus);
		}else if(r<(compare+=.01)){
			item = new Flashlight(qualityBonus);
		}
		
		return item;
	},
	drop: function(){
		if(inventory.selectedItemIndex!==null){
			var item = inventory.items[inventory.selectedItemIndex];

			if(item !== null){
				player.getTile().pushItem(item);
				inventory.items[inventory.selectedItemIndex]=null;
				inventory.selectedItemIndex = null;
				
				rconsole.log("You drop your "+item.getName()+".");
				
				rdetail.clear();
			}
		}
	},
	click: function(x,y){
		if(x>=inventory.x
		&& x<=inventory.x+inventory.itemsPerLine*64
		&& y>=inventory.y
		&& y<=inventory.y+(inventory.maxItems/inventory.itemsPerLine)*32){
			var itemIndex = Math.floor((y-inventory.y)/32)*inventory.itemsPerLine;
			itemIndex += Math.floor((x-inventory.x)/64);
			
			var previousIndex = inventory.selectedItemIndex;
			
			var selectedItem = inventory.items[itemIndex];
			if(selectedItem!==null){
				selectedItem = inventory.items[itemIndex];
				inventory.selectedItemIndex = itemIndex;
				
				var name = selectedItem.getName();
				var description = selectedItem.getDescription();
				
				rdetail.setItemDetail(name,description);
				
				if(previousIndex === inventory.selectedItemIndex){
					inventory.useSelectedItem();
				}
			}
		}
	}
}