map = {
	tiles: [],
	playerDrawX: 4,
	playerDrawY: 10,
	shortWidth: 200,
	longWidth: 1500,
	xWide: false,
	roadOffset: 5,
	getTile:function(x,y){
		var tile;
	
		if(!map.isWithinMap(x,y)){
			tile = new Tile();
			
			if(map.isRoad(x,y)){
				tile.pushBlock("road");
			}else if(map.isRoadAdjacent(x,y)){
				tile.pushBlock("snow");
			}else{
				tile.pushBlock("water");
			}
		}else{
			tile = map.tiles[x][y];
		}
		
		return tile;
	},
	isWithinMap:function(x,y){
		if(x<0 || y<0 || x>=map.width || y >= map.height){
			return false;
		}
		return true;
	},
	init:function(){
		map.mountains = [];
		map.forests = [];
		map.islands = [];
		map.rabbits = [];
		
		map.numMountains = 0;
		map.numRabbits = 40;
		map.numCabins = 160;
		
		map.numCaches = 100;
	
		map.islandWidth = map.shortWidth/4;
	
	
		if(game.roadDirection == 0){
			map.width = map.shortWidth;
			map.xWide = false;
			map.height = map.longWidth;
			
			map.isRoad = function(i,j){
				if(j >= map.roadOffset-1 && j <= map.roadOffset+1){
					return true;
				}
				return false;
			}
			map.isRoadAdjacent = function(i,j){
				if(j <= map.roadOffset+2 &&  (j == map.roadOffset-2 || j == map.roadOffset+2)){
					return true;
				}
				return false;
			}
		}else if(game.roadDirection ==1){
			map.width = map.longWidth;
			map.xWide = true;
			map.height = map.shortWidth;
			
			map.isRoad = function(i,j){
				if(i >= map.width - map.roadOffset-1 && i <= map.width - map.roadOffset+1){
					return true;
				}
				return false;
			}
			map.isRoadAdjacent = function(i,j){
				if(i >= map.width - map.roadOffset-2 &&  (i == map.width - map.roadOffset-2 || i == map.width - map.roadOffset+2)){
					return true;
				}
				return false;
			}
		}else if(game.roadDirection ==2){
			map.width = map.shortWidth;
			map.xWide = false;
			map.height = map.longWidth;
			
			
			
			map.isRoad = function(i,j){
				if(j >= map.height - map.roadOffset-1 && j <= map.height - map.roadOffset+1){
					return true;
				}
				return false;
			}
			map.isRoadAdjacent = function(i,j){
				if(j >= map.height - map.roadOffset-2 &&  (j == map.height - map.roadOffset-2 || j == map.height - map.roadOffset+2)){
					return true;
				}
				return false;
			}
		}else if(game.roadDirection ==3){
			map.width = map.longWidth;
			map.xWide = true;
			map.height = map.shortWidth;
			
			map.isRoad = function(i,j){
				if(i >= map.roadOffset-1 && i <= map.roadOffset+1){
					return true;
				}
				return false;
			}
			map.isRoadAdjacent = function(i,j){
				if(i <= map.roadOffset+2 &&  (i == map.roadOffset-2 || i == map.roadOffset+2)){
					return true;
				}
				return false;
			}
		}
		
	},
	updateRabbits:function(){
		for(var i=0; i<map.rabbits.length;i++){
			var rabbit = map.rabbits[i];
		
			//if near player
			var distance = Math.abs((rabbit.x-player.x)+(rabbit.y-player.y));
			
			var direction=Math.floor(Math.random()*4);
			
			var attemptMove = false;
			
			if(distance <10){
				direction = player.direction;
				attemptMove = true;
			}
		
		
			if(Math.random()>.5){
				attemptMove = true; 
			}
			
			//add tracks
			map.getTile(rabbit.x,rabbit.y).track = "rabbit";
			map.getTile(rabbit.x,rabbit.y).trackCounter = game.getTotalHours();
			
			if(attemptMove){
				if(direction == 0){
					if(map.rabbitSafe(rabbit.x,rabbit.y-1)){
						rabbit.y = rabbit.y-1;
					}
				}else if(direction ==1){
					if(map.rabbitSafe(rabbit.x+1,rabbit.y)){
						rabbit.x = rabbit.x+1;
					}
				}else if(direction == 2){
					
					if(map.rabbitSafe(rabbit.x,rabbit.y+1)){
						rabbit.y = rabbit.y+1;
					}
				}else{
					if(map.rabbitSafe(rabbit.x-1,rabbit.y)){
						rabbit.x = rabbit.x-1;
					}
				}
				
				
			}
		}
	},
	rabbitSafe:function(i,j){
		var tile = map.getTile(i,j);
		if (tile.getHeight()===32 && tile.getTopBlock().blockType !== "water"){
			return true;
		}
		return false;
	},
	getMountainHeight:function(x,y){
		var highest = -10000;
		var tempHeight;
		

		for(var i=0;i<map.mountains.length;i++){
			tempHeight = Math.floor(
				map.mountains[i].peak
				-Math.pow((x-map.mountains[i].x)*map.mountains[i].width,2)
				-Math.pow((y-map.mountains[i].y)*map.mountains[i].height,2)
			);
			
			if(tempHeight>highest){
				highest = tempHeight;
			}
		}
		return highest;
	},
	isInMountain:function(x,y){
		if(map.getMountainHeight(x,y) > -6){
			return true;
		}
		return false;
	},
	getIslandDistance:function(x,y){
		var closest = 10000;
		var tempDistance;

		for(var i=0;i<map.islands.length;i++){
			var tempDistance = Math.pow(
				Math.pow(x-map.islands[i].x,2)+Math.pow(y-map.islands[i].y,2)
			,.5);
		
			if(tempDistance<closest){
				closest = tempDistance;
			}
		}
		return closest;
	},
	addTree:function(tile,x,y){
		var treeHeight = 1+Math.floor(Math.random()*5);
		
		for(var i=0;i<=treeHeight;i++){
			tile.pushBlock("tree");
		}
	},
	generate: function(){
		for(var i=0; i<map.numMountains;i++){
			map.mountains.push({
				peak: 3+Math.floor(Math.random()*5),
				x:.1+.8*map.width,
				y:.1+.8*map.height,
				width:Math.random()*.7+.2,
				height:Math.random()*.7+.2
			});
		}
		
		var numIslands = map.longWidth/map.islandWidth;
		for(var i=0; i<numIslands;i++){
			if(map.xWide){
				map.islands.push({
					x:map.islandWidth*i,
					y:Math.floor(.1+Math.random()*.8*map.height)
				});
			}else{
				map.islands.push({
					x:Math.floor(.1+Math.random()*.8*map.width),
					y:map.islandWidth*i
				});
			}
		}
		map.islands.push({
			x:player.x,
			y:player.y
		});
		
		for(var i=0; i<map.numRabbits;i++){
			map.rabbits.push({
				x:Math.floor(.1+Math.random()*.8*map.width),
				y:Math.floor(.1+Math.random()*.8*map.height)
			});
		}
		
		
		
		
		
		
	
		for(var i=0; i<map.width;i++){
			map.tiles[i]=[];
			for(var j=0;j<map.height;j++){
				
				
				var newTile = new Tile();
		
				if(map.isRoad(i,j)){
					newTile.pushBlock("road");
				}else if(map.isRoadAdjacent(i,j)){
					newTile.pushBlock("snow");
				}else{
					var islandRatio = map.getIslandDistance(i,j)/map.islandWidth;
				

					
					
				
					if((2-islandRatio)<Math.random()){
						newTile.pushBlock("water");
						if((2-islandRatio)>Math.random()*.5){
							newTile.pushBlock("ice");
						}
					}else{
						var mountainHeight = map.getMountainHeight(i,j);
	
						if(mountainHeight > -6 && mountainHeight < 1){
							if(Math.random()<.4){
								newTile.pushBlock("snow");
							}else if(Math.random()<.45){
								newTile.pushBlock("dirt");
							}else{
								newTile.pushBlock("rock");
							}
						}else if(mountainHeight >=1){
							var mountainOffset = 0;
							if(Math.random()<.1){
								mountainOffset = 1;
							}
						
							for(var k=0;k<mountainHeight+mountainOffset;k++){
								if(Math.random()>.05){
									newTile.pushBlock("rock");
								}else{
									newTile.pushBlock("dirt");
								}
							}		
						}else{
							//base
							if(Math.random()>.95){
								newTile.pushBlock("water");
								if(Math.random()>.4){
									newTile.pushBlock("ice");
								}
							}else{
								newTile.pushBlock("snow");
								
								//add forest
							}		
						}  
					}
				}
		
				var blockType = newTile.getTopBlock().blockType;
				if(blockType !== "water" && blockType !== "road"){
					if(Math.random()>.95){
						map.addTree(newTile,i,j);
					}else{
						newTile.generateItems(.0125,0);
					}
				}
				
				
				
				/*
				
				if(map.isRoad(i,j)){
					newTile.pushBlock("road");
				}else if(map.isRoadAdjacent(i,j)){
					newTile.pushBlock("snow");
				}else{
								
					//base
					if(Math.random()>.8){
						newTile.pushBlock("water");
						if(Math.random()>.2){
							newTile.pushBlock("ice");
						}
					}else{
						newTile.pushBlock("snow");
						if(Math.random()>.95){
							newTile.pushBlock("tree");
							
							if(Math.random()>.5){
								newTile.pushBlock("tree");
							}
							if(Math.random()>.5){
								newTile.pushBlock("tree");
							}
						}				
					}
					
					
					for(var k=0;k<tileHeight;k++){
						//newTile.pushBlock(blockType);
					}
					
					if(newTile.getTopBlock().blockType !== "water"){
						newTile.generateItems(.03,0);
					}
				}
				
				
				*/
				
				
				
				map.tiles[i][j] = newTile;
				
			}
			
			
		
			//some generation code
			//for now, completely random
		}
		
		var cabinI;
		var cabinJ;
		
		var retries = 10*map.numCabins;
		
		for(var i=0;i<map.numCabins;i++){
			cabinI=Math.floor((Math.random()*.95+.025)*map.width);
			cabinJ=Math.floor((Math.random()*.95+.025)*map.height);
		
			if(!map.isInMountain(cabinI,cabinJ)
			&& !map.isRoadAdjacent(cabinI,cabinJ)
			&& !map.isRoad(cabinI,cabinJ)){
				map.generateCabin(
					cabinI,
					cabinJ
				);
			}else{
				if(retries>0){
					i--;
					retries--;
				}
			}
		}		
		
		
		
		var cacheI;
		var cacheJ;
		
		var retries = 10*map.numCaches;
		
		for(var i=0;i<map.numCaches;i++){
			cacheI=Math.floor((Math.random()*.95+.025)*map.width);
			cacheJ=Math.floor((Math.random()*.95+.025)*map.height);
		
			if(map.getTile(cacheI,cacheJ).getTopBlock().blockType === "snow"){
				var block = map.getTile(cacheI,cacheJ).pushBlock("cache");
				var description = block.getItemsDescription();
				
				var transponderRetries = 10;
				while(transponderRetries>0){
					var transponderI = Math.floor(cacheI+Math.random()*200-100)
					var transponderJ = Math.floor(cacheJ+Math.random()*200-100)
				
					if(map.getTile(transponderI,transponderJ).getTopBlock().blockType === "snow"){
						var transponder = new Transponder();
						transponder.cacheX = cacheI;
						transponder.cacheY = cacheJ;
						transponder.itemsDescription = description;
						map.getTile(transponderI,transponderJ).pushItem(transponder);
						break;
					}
					transponderRetries--;
				}
			}else{
				if(retries>0){
					i--;
					retries--;
				}
			}
		}
		
		
		
		var playerTile = new Tile();
		playerTile.pushBlock("snow");
		
		playerTile.pushItem(new Parachute());
		
		map.tiles[player.x][player.y] = playerTile;
		
		game.mapLoaded = true;
	},
	computeShadow: function(i,j,hour){
		var shadow=0;
		var shadowMag=0;
		var neighbor;
		
		var tile=map.getTile(i,j);
		
		if(hour>=20 || hour <= 04){
			shadow =.9;
		}else if(hour>=4 && hour<=6){
			shadow = .9-.45*((hour-4)/2);
		}else if(hour>=18 && hour<=20){
			shadow = .4+.45*((hour-18)/2);
		}else{
			if(hour>=6 && hour<=8){
				neighbor = map.getTile(i+1,j);
				shadowMag = .4-.25*((hour-6)/2);
			}else if(hour>=8 && hour<=10){
				neighbor = map.getTile(i+1,j);
				shadowMag = .15;
			}else if(hour>=10 && hour<=14){
				neighbor = map.getTile(i,j+1);
				shadowMag = .15;
			}else if(hour>=14 && hour<=16){
				neighbor = map.getTile(i-1,j);
				shadowMag = .15;
			}else if(hour>=16 && hour<=18){
				neighbor = map.getTile(i-1,j);
				shadowMag = .15+.25*((hour-16)/2);
			}
		
			var thisHeight = tile.getHeight();
			var neighborHeight = neighbor.getHeight();
			
			if(thisHeight<neighborHeight){
				shadow=shadowMag;
			}
		}
				
		return shadow;
	},
	computeWind:function(i,j){
		var wind = 0;
		
		var potentialWind = temp.wind/100;
		var windDirection = temp.windDirection;
		
		var tile=map.getTile(i,j);
		var neighbor;
		
		if(windDirection===0){
			neighbor = map.getTile(i,j-1);
		}else if(windDirection===1){
			neighbor = map.getTile(i-1,j);
		}else if(windDirection==2){
			neighbor = map.getTile(i,j+1);
		}else if(windDirection==3){
			neighbor = map.getTile(i+1,j);
		}
		
		var thisHeight = tile.getHeight();
		var neighborHeight = neighbor.getHeight();
		
		if(thisHeight>=neighborHeight){
			wind=potentialWind;
		}
		
		return wind;
	},
	draw: function(ctx, img, bufferCanvas, bufferCtx){
		var tile, blocks, realI, realJ;
	
		var hour = game.getHour();
		var windDrawDirection = (temp.windDirection+player.direction)%4;
		
		var flashLightEquipped = false
		var binocularsEquipped = false;
		
		var items = inventory.items;
		for(var i=0;i<items.length;i++){
			if(items[i]!==null && items[i].equipped){
				if(items[i].getName()==="flashlight"){
					flashLightEquipped = true;
				}else if(items[i].getName()==="binoculars"){
					binocularsEquipped = true;
				}
			}
		}
	
		var binocularsOffset = 0;
		if(binocularsEquipped){
			binocularsOffset=10;
		}
	
		for(var i=0;i<9;i++){
			for(var j=-3;j<12;j++){
				if(!binocularsEquipped && i==4 && j==11){
					ctx.globalAlpha = .4;
				}else{
					ctx.globalAlpha = 1;
				}
			
				var blockHeight = 0;
				
				if(player.direction == 0){
					realI = player.x+(i-4);
					realJ = player.y+(j-10)-binocularsOffset;
				}else if(player.direction == 1){
					realI = player.x-(j-10)+binocularsOffset;
					realJ = player.y+(i-4);
				}else if(player.direction == 2){
					realI = player.x-(i-4);
					realJ = player.y-(j-10)+binocularsOffset;
				}else if(player.direction == 3){
					realI = player.x+(j-10)-binocularsOffset;
					realJ = player.y-(i-4);
				}else{
					throw "Invalid direction!";
				}
				
			
				tile = map.getTile(realI,realJ);
				blocks = tile.blocks;
				items = tile.items;
				
				shadow = map.computeShadow(realI,realJ,hour);
				
				var realShadow;
				if(flashLightEquipped && i==4 && j<map.playerDrawY && !binocularsEquipped){
					realShadow = Math.min(.5+.04*(j),shadow);
				}else{
					realShadow = shadow;
				}
				
				for(var k=0;k<blocks.length;k++){
					blockHeight+=blocks[k].height;
					
					bufferCtx.clearRect(0,0,64,64);
					bufferCtx.globalCompositeOperation="source-over";
					
					var blockX = blocks[k].ssX;
					var blockY = blocks[k].ssY;
					if(tile.canHallucinate<player.hallucinate){
						blockX=112;
						blockY=0;
					}
					bufferCtx.drawImage(img,blockX,blockY,16,16,0,0,64,64);
					
					//neighbor to check:
					if(realShadow>0){
						bufferCtx.globalCompositeOperation="source-atop";
						bufferCtx.fillStyle = "rgba(5,0,15,"+realShadow+")";
						bufferCtx.fillRect(0,0,64,64);
					}
					
					ctx.drawImage(bufferCanvas, i*64,(j)*32-blockHeight+32+(player.z-32));
				}
				
				
				
				//TRACKS
				var tempAlpha = ctx.globalAlpha;
				if(tile.getTopBlock().blockType==="snow" && tile.track!==null){
					ctx.globalAlpha = Math.max(0,1-(game.getTotalHours() - tile.trackCounter)/12);
				
					bufferCtx.clearRect(0,0,64,64);
					bufferCtx.globalCompositeOperation="source-over";
					
					if(tile.track==="rabbit"){
						bufferCtx.drawImage(img,64,48,16,8,0,0,64,32);
					}else if(tile.track==="player"){
						bufferCtx.drawImage(img,64,56,16,8,0,0,64,32);
					}
					
					//neighbor to check:
					if(realShadow>0){
						bufferCtx.globalCompositeOperation="source-atop";
						bufferCtx.fillStyle = "rgba(5,0,15,"+realShadow+")";
						bufferCtx.fillRect(0,0,64,64);
					}
					
					ctx.drawImage(bufferCanvas, i*64,(j)*32-blockHeight+32+(player.z-32));
				}
				
				ctx.globalAlpha = tempAlpha;
				
				
				
				//shade!
				
				//6-8
				
				//0  X
				//  -N-
				//Y E-W
				//  -S-
				
				
      			
				
				
				/*ctx.globalCompositeOperation = "source-in";
				
				if(Math.random()<.5){
					ctx.fillStyle = "rgba(0,0,20,.7)";
					ctx.fillRect(i*64,(j)*32-blockHeight+32+(player.z-32),64,32);
				}*/
								
				// 06-08 dark shade from E->W
				// 08-10 light shade from E->W
				// 10-14 light shade from S->N
				// 14-16 light shade from W->E
				// 16-18 dark shade from W->E
				//night....
				// 18-20 moderately dark everywhere
				// 20-04 super dark everywhere
				// 04-6 moderately dark everywhere
				
				
				
				
				//draw items
				for(var k=0;k<items.length;k++){
					if(items[k]!==null){
						items[k].draw(ctx,img,tile,i,j,bufferCanvas,bufferCtx,realShadow);
					}
				}
				
				
				//RABBITS
				for(var k=0;k<map.rabbits.length;k++){
					if(map.rabbits[k].x===realI && map.rabbits[k].y===realJ){
						bufferCtx.clearRect(0,0,64,64);
						bufferCtx.globalCompositeOperation="source-over";
						bufferCtx.drawImage(img,80,56,16,8,0,0,64,32);	
						
						//neighbor to check:
						if(realShadow>0){
							bufferCtx.globalCompositeOperation="source-atop";
							bufferCtx.fillStyle = "rgba(5,0,15,"+realShadow+")";
							bufferCtx.fillRect(0,0,64,64);
						}
						ctx.drawImage(bufferCanvas, i*64,(j)*32-blockHeight+32+(player.z-32));
					}
				}
				
				if(!binocularsEquipped){
					//DRAW PLAYER
					if(map.playerDrawX==i && map.playerDrawY==j){
						var waterOffset=0;
						if(player.inWater()){
							waterOffset=48;
						}
						
						//draw shadow
						//draw skin
						//daw underwear
						//draw hair
						ctx.drawImage(img,16,80,16,16,map.playerDrawX*64,map.playerDrawY*32-40+waterOffset,64,64);
						ctx.drawImage(img,16*player.skinColor,96,16,16,map.playerDrawX*64,map.playerDrawY*32-40+waterOffset,64,64);
						ctx.drawImage(img,16*player.underwear,120,16,16,map.playerDrawX*64,map.playerDrawY*32-40+waterOffset,64,64);
						ctx.drawImage(img,16*player.hairColor,112,16,8,map.playerDrawX*64,map.playerDrawY*32-40+waterOffset,64,32);
						
						//draw equipment
						var iItems = inventory.items;
						for(var l=0;l<iItems.length;l++){
							if(iItems[l]!==null){
								iItems[l].drawEquipped(ctx,img,waterOffset);
							}
						}
					}
				}
				
				var wind = map.computeWind(realI,realJ);
				
				//draw wind
				bufferCtx.clearRect(0,0,64,64);
				bufferCtx.globalCompositeOperation="source-over";
				bufferCtx.drawImage(img,16*(game.counter%4),48+8*windDrawDirection,16,8,0,0,64,32);	
				
				bufferCtx.globalCompositeOperation="source-atop";
				bufferCtx.fillStyle = "rgba(5,0,15,"+realShadow+")";
				bufferCtx.fillRect(0,0,64,64);
				
				
				ctx.globalAlpha = wind;
				ctx.drawImage(bufferCanvas, i*64,(j)*32-blockHeight+16+(player.z-32));
				ctx.globalAlpha = 1;
			}
		}
		
		//draw player
		
		//ctx.drawImage(img,0,16,16,16,map.playerDrawX*64,map.playerDrawY*32-40,64,64);
		
	},
	generateCabin:function(i,j){
		var wallHeight=2;
		
		var totalI,totalJ,tile;
		
		for(var k=-3;k<=3;k++){
			for(var l=-3;l<=3;l++){
				totalI = i+k;
				totalJ = j+l;
				
				if(map.isWithinMap(totalI,totalJ)){
					tile = new Tile();
					
					if(k===-3 || k===3 || l===-3 || l===3){
						tile.pushBlock("snow");
					}else{
						tile.pushBlock("logfloor");
		
						if(k==0 && l==-2){
							for(var n=0;n<wallHeight;n++){
								tile.pushBlock("door");
							}
						}else if(k==-2 || k==2 || l ==2 || l==-2){
							for(var n=0;n<wallHeight;n++){
								tile.pushBlock("log");
							}
						}else{
							tile.generateItems(.5,2);
						}
					}
					
					map.tiles[totalI][totalJ] = tile;
				}
			}
		}
		
		
		//USE THE BELOW FOR NOTES?
		
		/*
		//draw key within 50 tiles
		var keyTries = 10;
		var keyPlaced = false;
		var keyMag;
		while(keyTries>0 && !keyPlaced){
			if(Math.random()>.5){
				keyMag = 1;
			}else{
				keyMag = -1;
			}
			totalI = Math.floor(Math.random*5+10)*keyMag+i;
			if(Math.random()>.5){
				keyMag = 1;
			}else{
				keyMag = -1;
			}
			totalJ = Math.floor(Math.random*5+10)*keyMag+j;
			//pick a tile
			//check to see if it is valid
				//and not water
			
			//put a key there
			
			if(map.
			
			keyTries--;
		}
		
		*/
	}
}