player = {
	deathTemp: 75,
	startingTemp: 98,
	regenRate: .06,
	startingHunger: 50,
	hungerRate: .10,
	startOffset: 100,
	startingWeight: 100,
	getShadow: function(){
		return map.computeShadow(
			player.x,
			player.y,
			game.getHour()
		);
	},
	getWind: function(){
		return map.computeWind(
			player.x,
			player.y
		);
	},
	hasWatch: function(){
		var items = inventory.items;
		for(var i=0;i<items.length;i++){
			if(items[i]!== null && items[i].name==="watch"){
				return true;
			}
		}
		return false;
	},
	hasCompass: function(){
		var items = inventory.items;
		for(var i=0;i<items.length;i++){
			if(items[i]!== null && items[i].name==="compass"){
				return true;
			}
		}
		return false;
	},
	init:function(){
		player.hallucinate = 0;
	
		if(game.roadDirection == 0){
			player.x = Math.floor((Math.random()*.6+.2)*map.width);
			player.y = map.height - player.startOffset;
		}else if(game.roadDirection == 1){
			player.x = player.startOffset;	
			player.y = Math.floor((Math.random()*.6+.2)*map.height);
		}else if(game.roadDirection == 2){
			player.x = Math.floor((Math.random()*.6+.2)*map.width);
			player.y = player.startOffset;
		}else if(game.roadDirection == 3){
			player.x = map.width - player.startOffset;
			player.y = Math.floor((Math.random()*.6+.2)*map.height);
		}
		
		player.z = 32;
		player.direction= Math.floor(Math.random()*4);
		
		player.coreTemp = player.startingTemp;
		player.hunger = player.startingHunger;
		
		player.movesAndTurns = 0;
		player.moves = 0;
		player.foodEaten = 0;
		
		player.hoursAwake = 0;
		
		player.sleepColdResistance = 0;
		player.sleepWindResistance = 0;
		
		
		player.skinColor = Math.floor(Math.random()*5);
		player.underwear = Math.floor(Math.random()*2);
		player.hairColor = Math.floor(Math.random()*6);
		
	},
	sleep: function(cold,wind){
		//or.... allow sleep if really cold
	
		player.sleepColdResistance = cold;
		player.sleepWindResistance = wind;
		
		if(player.inCabin()){
			player.sleepColdResistance = 1-(1-cold)*(1-.8);
			player.sleepWindResistance = 1-(1-wind)*(1-.95);
		}
		
		var sleepExertion = .25;
	
		if(player.hoursAwake < 6 && player.coreTemp>=80){
			rconsole.log("You're not tired enough to sleep.");
		}else{
			var hoursToSleep = 6 + Math.floor(player.hoursAwake*(.125+.125*Math.random()));
		
			for(var i=0;i<hoursToSleep;i++){
				if(!game.isDead){
					game.updateTime(game.timePerHour, .25);
				}else{
					return;
				}
			}
			
			player.hoursAwake = 0;
			player.direction= Math.floor(Math.random()*4);
			
			rconsole.log("You sleep.");
		}

		
		
		player.sleepColdResistance = 0;
		player.sleepWindResistance = 0;
	},
	inCabin:function(){
		if(player.getTile().getTopBlock().blockType === "logfloor"){
			return true;
		}
		return false;
	},
	inWater: function(){
		if(player.getTile().getTopBlock().blockType === "water"){
			return true;
		}
		return false;
	},
	updateHunger:function(step,exertion){
		player.hunger -= step*player.hungerRate*exertion;

		if(player.hunger < 0){
			player.die("starvation");
		}else{
			if(player.hunger<10){
				rconsole.logNoBreak('<div class="help">You are starving!</div>');
			}
		}
	},
	updateCoreTemp: function(step, exertion){	
		var clothingWetness = inventory.getEquippedWetness();

		var coldLoss=(70-temp.temp)*(1/600)*(1-inventory.getTotalColdResistance())*(1-player.sleepColdResistance);
		var windLoss = .20*player.getWind()*(1-inventory.getTotalWindResistance())*(1-player.sleepWindResistance);
		var regen = player.regenRate*exertion;
		var shadeBonus = Math.max(0,.5-player.getShadow())*.1;

		var totalLost = step * (
			coldLoss
			+ ((player.inWater())?2:0)
			+ clothingWetness*2
			+ windLoss
			- regen
			- shadeBonus
		);

		player.coreTemp = Math.min(98,player.coreTemp-totalLost);
		
		/*
		console.log("COLD LOSS",coldLoss);
		console.log("WIND LOSS",windLoss);
		console.log("REGEN BONUS",regen);
		console.log("SHADE BONUS",shadeBonus);
		console.log("TOTAL LOST", totalLost);
		console.log("======================");
		*/
		
		if(player.coreTemp < player.deathTemp){
			//you lose! :(
			
			player.die("hypothermia");
		}else{
			if(player.coreTemp<80){
				rconsole.logNoBreak('<div class="help">You are freezing!</div>');
				
				if(Math.random()<.05){
					rconsole.logNoBreak('<div>Maybe you should sleep... just for a few minutes.</div>');
				}
			}
			
			if(player.coreTemp<85 && player.coreTemp >80){
				var r= Math.random();
				if(r<.1){
					rconsole.logNoBreak('<div>Maybe it\'s not so cold. You actually feel really hot.</div>');
				
					//paradoxical undressing
					if(r<.01){
						var items = inventory.items;
						for(var i=0;i<items.length;i++){
							if(items[i]!==null && items[i].isClothing && items[i].equipped){
								items[i].use();
								break;
							}
						}
					}
				}
			}
		}
	},
	disrobe:function(){
		var takenOff = 0;
		var items=inventory.items;
		for(var i=0;i<items.length;i++){
			if(items[i]!==null && items[i].isClothing && items[i].equipped){
				items[i].use();
				takenOff++;
			}
		}
		
		rconsole.log("You took off "+takenOff+" pieces of clothing.");
	},
	die: function(cause){
		//return;
	
		game.isDead = true;
		rconsole.log("======================");
		rconsole.logNoBreak('<div class="lose">You died of '+cause+'.</div>');
		rconsole.log("Press 'y' to start a new game.");
		player.listExploits();
	},
	win: function(){
		game.isDead = true;
		rconsole.logNoBreak('<div class="win">You win.</div>');
		rconsole.log("Press 'y' to start a new game.");
		player.listExploits();
		
		//credits
		rconsole.log("----------------------");
		rconsole.logNoBreak('<div>A game by Jeremiah Reid</div>');
		rconsole.logNoBreak('<div>Special thanks: Arjun Aneja, Daniel Clark, Joshua Martin, Nicole Reid, Matt Wilson</div>');
	},
	listExploits: function(){
		rconsole.log("----------------------");
		rconsole.log("You took "+player.moves+" moves.");
		rconsole.log("You ate "+player.foodEaten+" pieces of food.");
		rconsole.log("You lasted "+Math.floor(game.getTotalHours())+" hours.");
		rconsole.log("Your core body temperature was "+temp.formatTemp(player.coreTemp)+".");
	},
	getFormattedCoreTemp: function(){
	
	},
	drawPlayerSummary: function(ctx){
		//draw cold bar
		//draw total resistance
		
		ctx.font= "12px Verdana";
		
		ctx.fillStyle = "rgba(255,255,255,1)";
		ctx.fillText("Cold resistance: " + Math.floor(inventory.getTotalColdResistance()*1000)/10 +"%", 624,380);
		
		ctx.fillText("Wind resistance: " + Math.floor(inventory.getTotalWindResistance()*1000)/10 +"%", 624,400);
		
		ctx.fillText("Weight: " + player.getTotalFormattedWeight(), 624,420);
		
		ctx.fillText("Core Body Temperature", 624,450);
		
		ctx.fillText("Satiety", 624,485);
		
		var tempBarValue = Math.max(0,(player.coreTemp-player.deathTemp)/(player.startingTemp-player.deathTemp));
		ctx.fillStyle = "rgba(200,50,50,1)";
		ctx.fillRect(624,455,300*tempBarValue,12);
		
		tempBarValue = Math.max(0,player.hunger/player.startingHunger);
		ctx.fillStyle = "rgba(50,200,50,1)";
		ctx.fillRect(624,490,300*tempBarValue,12);
	},
	getTotalFormattedWeight:function(){
		var weight = 0;
		
		var items = inventory.items;
		for(var i=0;i<items.length;i++){
			if(items[i]!==null){
				weight+=items[i].weight;
			}
		}
		
		return Math.floor(weight*10)/10 + "lb";
	},
	getTotalWeight:function(){
		var weight = 0;
		
		var items = inventory.items;
		for(var i=0;i<items.length;i++){
			if(items[i]!==null){
				weight+=items[i].weight;
			}
		}
		
		return weight;
	},
	getWeightRatio:function(){
		return (player.startingWeight+player.getTotalWeight())/player.startingWeight;
	},
	moveForward:function(){
		tempX=player.x;
		tempY=player.y;
		
		if(player.direction == 0){
			tempY--;
		}else if(player.direction == 1){
			tempX++;
		}else if(player.direction == 2){
			tempY++;
		}else{
			tempX--;
		}		
		
		player.move(tempX,tempY);
	},
	moveBack:function(){
		tempX=player.x;
		tempY=player.y;
	
		if(player.direction == 0){
			tempY++;
		}else if(player.direction == 1){
			tempX--;
		}else if(player.direction == 2){
			tempY--;
		}else{
			tempX++;
		}
		
		player.move(tempX,tempY);
	},
	getTile:function(){
		return map.getTile(player.x,player.y);
	},
	move:function(x,y){
		var tile = map.getTile(x,y);
		var playerHeight = player.getTile().getHeight();
		var tempHeight = tile.getHeight();

		//todo: add pick axe for climbing, also add fall death
		
		//pass a reference to tile
		
		if(tile.getTopBlock().climb(tile)){
			if(tempHeight <= playerHeight+32){
			
				player.movesAndTurns++;
			
				player.x=x;
				player.y=y;
				player.z=tempHeight;
				
				var items = tile.items;
				var item;
				for(var i=0;i<items.length;i++){
					item = items[i];
					if(item!== null && !item.previouslyPickedUp){
						if(inventory.pickUp(item)){
							item.previouslyPickedUp = true;
							
							//remove from ground
							items[i]=null;
						}else{
							rconsole.log("There is a "+item.getName()+" here.");
						}
					}
					
				}
				
				player.getTile().getTopBlock().stepOn();
				
				player.getTile().track = "player";
				player.getTile().trackCounter = game.getTotalHours();
				
				//todo: add running, etc.
				var speed = 1;
				if(player.inWater()){
					speed*=.5;
				}
				
				var playerWeightRatio=player.getWeightRatio();
				
				speed *= (1/playerWeightRatio);
				
				var exertion = Math.pow(playerWeightRatio,2);
				
				//TIME
				game.updateTime(1/speed, exertion);
				
				player.moves++;
			}else{
				rconsole.log("That is too high to climb.");
			}
		}		
		//rconsole.log(player.getTile().getTopBlock().blockType);
	},
	wait:function(){
		game.updateTime(1, .5);
		rconsole.log('You wait.');
	},
	turnLeft:function(){
		player.movesAndTurns++;
		player.direction = (player.direction+3)%4;
	},
	turnRight:function(){
		player.movesAndTurns++;
		player.direction = (player.direction+1)%4;
	}
}