function Tile(){
	this.blocks = [];
	this.items = [];
	this.shadow=0;
	
	this.blockCounter = 0;
	
	this.track = null;
	this.trackCounter = 0;
	
	this.canHallucinate = Math.random()*275+25;
}

Tile.prototype.pushBlock = function(blockType){
	var block;

	this.blockCounter++;

	if(blockType == "water"){
		block = new Water(this.blockCounter);
	}else if(blockType == "ice"){
		block = new Ice(this.blockCounter);
	}else if(blockType == "snow"){
		block = new Snow(this.blockCounter);
	}else if(blockType == "tree"){
		block = new Tree(this.blockCounter);
	}else if(blockType == "dirt"){
		block = new Dirt(this.blockCounter);
	}else if(blockType == "leaves"){
		block = new Leaves(this.blockCounter);
	}else if(blockType == "rock"){
		block = new Rock(this.blockCounter);
	}else if(blockType == "checker"){
		block = new Checker(this.blockCounter);
	}else if(blockType == "road"){
		block = new Road(this.blockCounter);
	}else if(blockType == "log"){
		block = new Log(this.blockCounter);
	}else if(blockType == "door"){
		block = new Door(this.blockCounter);
	}else if(blockType == "logfloor"){
		block = new LogFloor(this.blockCounter);
	}else if(blockType == "cache"){
		block = new Cache(this.blockCounter);
	}else{
		throw("Invalid block type");
	}

	block.blockType = blockType;

	this.blocks.push(block);
	
	return block;
}

Tile.prototype.getTopBlock=function(){
	return this.blocks[this.blocks.length-1];
}

Tile.prototype.generateItems = function(rate,qualityBonus){
	
	
	//generate more items if this is a cabin
	
	var item = null;
	
	if(Math.random()<rate){	
		item = inventory.itemRoll(qualityBonus);
	}
		
	if(item!=null){
		this.pushItem(item);
	}
}

Tile.prototype.pushItem=function(item){
	this.items.push(item);
}

Tile.prototype.getHeight=function(){
	var blockHeight = 0;
	for(var i=0;i<this.blocks.length;i++){
		blockHeight+=this.blocks[i].height;
	}
	return blockHeight;
}

function Water(){
	this.ssX = 16;
	this.ssY = 0;
	this.height = 32;
}
Water.prototype.stepOn=function(){
	rconsole.logNoBreak('<div class="noise">splash</div>');
}
Water.prototype.climb=function(){
	return true;
}

function Ice(){
	this.ssX = 0;
	this.ssY = 0;
	this.height = 0;
	
	this.weakness = Math.random()*.45+.05;
}
Ice.prototype.stepOn=function(){
	this.weakness +=.1*player.getWeightRatio();

	rconsole.logNoBreak('<div class="noise">crack</div>');
	
	var waterNeighbors = 0;
	//check neighbors
	for(var i=-1;i<=1;i++){
		for(var j=-1;j<=1;j++){
			if(!(i==0 && j==0)){
				if(map.getTile(player.x+i,player.y+j).getTopBlock().blockType === "water"){
					waterNeighbors++;
				}
			}
		}
	}
	
	var totalWeakness = this.weakness*((waterNeighbors+2)/10);
	
	if(Math.random()<totalWeakness){
		var blocks = player.getTile().blocks;
		
		//DELETE ICE
		blocks.splice(-1,1);
		
		player.getTile().getTopBlock().stepOn();
	}
}
Ice.prototype.climb=function(){
	return true;
}

function Snow(){
	this.ssX = 32;
	this.ssY = 0;
	this.height = 32;
}
Snow.prototype.stepOn=function(){
	rconsole.logNoBreak('<div class="noise">crunch</div>');
}
Snow.prototype.climb=function(){
	return true;
}


function Dirt(){
	this.ssX = 48;
	this.ssY = 0;
	this.height = 32;
}
Dirt.prototype.stepOn=function(){

}
Dirt.prototype.climb=function(){
	return true;
}



function Leaves(){
	this.ssX = 64;
	this.ssY = 0;
	this.height = 0;
}
Leaves.prototype.stepOn=function(){
	rconsole.logNoBreak('<div class="noise">crunch</div>');
}
Leaves.prototype.climb=function(){
	return true;
}


function Rock(counter){
	if(counter%2===0){
		this.ssX = 128;
		this.ssY = 64;
	}else{
		this.ssX = 128;
		this.ssY = 80;
	}
	
	this.height = 32;
}
Rock.prototype.stepOn=function(){

}
Rock.prototype.climb=function(){
	return true;
}
Rock.prototype.climb=function(){
	return true;
}



function Tree(){
	this.ssX = 96;
	this.ssY = 0;
	this.height = 32;
}
Tree.prototype.stepOn=function(){

}
Tree.prototype.climb=function(){
	return true;
}


function Checker(){
	this.ssX = 112;
	this.ssY = 0;
	this.height = 32;
}
Checker.prototype.stepOn=function(){

}
Checker.prototype.climb=function(){
	return true;
}

function Road(){
	this.ssX = 128;
	this.ssY = 0;
	this.height = 32;
}
Road.prototype.stepOn=function(){
	//you win! :)
	player.turnLeft();
	player.win();
}
Road.prototype.climb=function(){
	return true;
}

function Log(){
	this.ssX = 128;
	this.ssY = 16;
	this.height = 32;
}
Log.prototype.stepOn=function(){
	
}
Log.prototype.climb=function(){
	return false;
}

function LogFloor(){
	this.ssX = 128;
	this.ssY = 48;
	this.height = 32;
}
LogFloor.prototype.stepOn=function(){
	
}
LogFloor.prototype.climb=function(){
	return true;
}

function Door(){
	this.ssX = 128;
	this.ssY = 32;
	this.height = 32;
}
Door.prototype.stepOn=function(){
	
}
Door.prototype.climb=function(parentTile){
	//loop through all items
	var items = inventory.items;
	for(var i=0;i<items.length;i++){
		if(items[i]!==null && items[i].equipType === "key"){
			var blocks = parentTile.blocks;
		
			//DELETE DOOR (2 high)
			blocks.splice(-2,2);
			
			items[i]=null;

			return false;
		}
	}
	
	rconsole.log("You'll need a key for that.");
	return false;
}




function Cache(){
	this.ssX = 64;
	this.ssY = 72;
	this.height = 32;
	
	this.cacheItems = [];
	for(var i=0;i<5;i++){
		var item = inventory.itemRoll(3);
		if(item !== null){
			this.cacheItems.push(item);
		}
	}
}
Cache.prototype.stepOn=function(){
	
}
Cache.prototype.climb=function(parentTile){
	//loop through all items
	var items = inventory.items;
	for(var i=0;i<items.length;i++){
		if(items[i]!==null && items[i].equipType === "key"){
			var blocks = parentTile.blocks;
		
			//ADD NEW ITEMS HERE
			for(var j=0;j<this.cacheItems.length;j++){
				if(this.cacheItems[j] !== null){
					parentTile.pushItem(this.cacheItems[j]);
				}
			}
		
			//DELETE CACHE (1 high)
			blocks.splice(-1,1);
			
			items[i]=null;

			return false;
		}
	}
	
	rconsole.log("You'll need a key for that.");
	return false;
}

Cache.prototype.getItemsDescription=function(){
	var description = "";
	
	for(var i=0;i<this.cacheItems.length;i++){
		description += '<div> RF0000'+(i+1)+' - '+this.cacheItems[i].getName()+'</div>';
	}
	return description;
}
